#include "snd.h"

static int remove_temp_files(chan_info *cp, void *ptr)
{
  free_sound_list(cp);
  free_mix_list(cp);
  return(0);
}

void snd_exit_cleanly(snd_state *ss)
{  
  CLM_disconnect(ss);
  SND_disconnect(ss);
  cleanup_clm_dialog(ss);
  cleanup_dac();
  if (ss->to_clm)
    {
      close(ss->from_clm);
      ss->from_clm = 0;
    }
  if (ss->to_clm)
    {
      close(ss->to_clm);
      ss->to_clm = 0;
    }
  map_over_chans(ss,remove_temp_files,NULL);
  cleanup_region_temp_files();
  cleanup_group_temp_files(ss);
  cleanup_recording();
}

#define SAVE_BUF_SIZE 1024
static char save_buf[SAVE_BUF_SIZE];

static void save_snd_state_options (snd_state *ss, int fd)
{ /* for save options menu choice (.snd) -- mostly saving snd_state info */
  time_t ts;
  sprintf(save_buf,"\n;;; snd: ");
  write(fd,save_buf,strlen(save_buf));
#if (!defined(BEOS)) && (!defined(HAVE_CONFIG_H)) || defined(HAVE_STRFTIME)
  time(&ts);
  strftime(save_buf,SAVE_BUF_SIZE,"%a %d-%b-%y %H:%M %Z",localtime(&ts));
  write(fd,save_buf,strlen(save_buf));
#endif
  write(fd,"\n",1);
  sprintf(save_buf,"(set-show-fft-peaks %d)\n(set-show-marks %d)\n(set-show-y-zero %d)\n",
	  show_fft_peaks(ss),
	  show_marks(ss),
	  show_y_zero(ss));
  write(fd,save_buf,strlen(save_buf));
  sprintf(save_buf,"(set-fft-log-magnitude %d)\n(set-fft-log-frequency %d)\n(set-graph-style %d)\n(set-subsampling %d)\n",
	  fft_log_magnitude(ss),
	  fft_log_frequency(ss),
	  graph_style(ss),
	  subsampling(ss));
  write(fd,save_buf,strlen(save_buf));
  sprintf(save_buf,"(set-window-width %d)\n(set-window-height %d)\n(set-save-state-on-exit %d)\n",
	  snd_window_width(ss),snd_window_height(ss),save_state_on_exit(ss));
  write(fd,save_buf,strlen(save_buf));
  sprintf(save_buf,"(set-fft-size %d)\n(set-fft-window %d)\n(set-fft-style %d)\n(set-spectro-cutoff %.4f)\n",
	  fft_size(ss),
	  fft_window(ss),
	  fft_style(ss),
	  spectro_cutoff(ss));
  write(fd,save_buf,strlen(save_buf));
  if (fit_data_on_open(ss) != -1)
    {
      sprintf(save_buf,"(set-fit-data-on-open %d)\n",fit_data_on_open(ss));
      write(fd,save_buf,strlen(save_buf));
    }
  sprintf(save_buf,"(set-zoom-focus-style %d)\n(set-verbose-cursor %d)\n(set-color-scale %.4f)\n(set-color-inverted %d)\n(set-color-cutoff %.4f)\n",
	  zoom_focus_style(ss),
	  verbose_cursor(ss),
	  color_scale(ss),
	  color_inverted(ss),
	  color_cutoff(ss));
  write(fd,save_buf,strlen(save_buf));
  sprintf(save_buf,"(set-spectro-hop %d)\n(set-spectro-x-angle %.4f)\n(set-spectro-y-angle %.4f)\n(set-spectro-z-angle %.4f)\n",
	  spectro_hop(ss),spectro_x_angle(ss),spectro_y_angle(ss),spectro_z_angle(ss));
  write(fd,save_buf,strlen(save_buf));
  sprintf(save_buf,"(set-spectro-x-scale %.4f)\n(set-spectro-y-scale %.4f)\n(set-spectro-z-scale %.4f)\n(set-wavo-hop %d)\n(set-wavo-trace %d)\n",
	  spectro_x_scale(ss),spectro_y_scale(ss),spectro_z_scale(ss),wavo_hop(ss),wavo_trace(ss));
  write(fd,save_buf,strlen(save_buf));
  sprintf(save_buf,"(set-dot-size %d)\n(set-spectro-color %d)\n(set-channel-style %d)\n",
	  dot_size(ss),spectro_color(ss),channel_style(ss));
  write(fd,save_buf,strlen(save_buf));
  sprintf(save_buf,"(set-default-amp %.4f)\n(set-default-contrast %.4f)\n(set-default-contrast-amp %.4f)\n(set-default-contrasting %d)\n",
	  default_amp(ss),default_contrast(ss),default_contrast_amp(ss),default_contrasting(ss));
  write(fd,save_buf,strlen(save_buf));
  sprintf(save_buf,"(set-default-expand %.4f)\n(set-default-expanding %d)\n(set-default-expand-hop %.4f)\n(set-default-expand-length %.4f)\n",
	  default_expand(ss),default_expanding(ss),default_expand_hop(ss),default_expand_length(ss));
  write(fd,save_buf,strlen(save_buf));  
  sprintf(save_buf,"(set-default-expand-ramp %.4f)\n(set-default-filtering %d)\n(set-default-filter-order %d)\n(set-default-reverbing %d)\n",
	  default_expand_ramp(ss),default_filtering(ss),default_filter_order(ss),default_reverbing(ss));
  write(fd,save_buf,strlen(save_buf));
  sprintf(save_buf,"(set-reverb-decay %.4f)\n(set-default-reverb-feedback %.4f)\n(set-default-reverb-length %.4f)\n",
	  reverb_decay(ss),default_reverb_feedback(ss),default_reverb_length(ss));
  write(fd,save_buf,strlen(save_buf));
  sprintf(save_buf,"(set-default-reverb-lowpass %.4f)\n(set-default-reverb-scale %.4f)\n(set-default-speed %.4f)\n",
	  default_reverb_lowpass(ss),default_reverb_scale(ss),default_speed(ss));
  write(fd,save_buf,strlen(save_buf));
  sprintf(save_buf,"(set-line-size %d)\n(set-ask-before-overwrite %d)\n(set-wavo %d)\n(set-wavelet-type %d)\n",
	  line_size(ss),ask_before_overwrite(ss),wavo(ss),wavelet_type(ss));
  write(fd,save_buf,strlen(save_buf));
  sprintf(save_buf,";;; end option save\n");
  write(fd,save_buf,strlen(save_buf));
}

static int open_restart_file(char *name, int append)
{
  int fd;
  char *str;
  str = name;
  if ((*str) == '~')
    {
      strcpy(save_buf,getenv("HOME"));
      strcat(save_buf,++str);
      str=save_buf;
    }
  if (append)
    {
      if ((fd = open(str,O_RDWR,0)) == -1)
	fd = creat(str,0666);
      else
	lseek(fd,0L,2); /* go to the end */
    }
  else
    fd = creat(str,0666);
  return(fd);
}

int open_snd_init_file (snd_state *ss)
{ /* needed also by keyboard macro saver */
  return(open_restart_file(ss->init_file,TRUE));
}

void save_snd_options (snd_state *ss)
{
  int fd;
  fd = open_snd_init_file(ss);
  save_snd_state_options(ss,fd);
  close(fd);
}

int save_sound_state (snd_info *sp, void *ptr) 
{
  int chan,fd;
  chan_info *cp;
  axis_info *ap;
  fd = (*(int *)ptr);
  sprintf(save_buf,"(%s %s)\n  (set-syncing %d)\n  (set-contrasting %d) (set-contrast %.4f)\n  (set-expanding %d) (set-expand %.4f)\n",
	  (sp->read_only) ? "view-sound" : "open-sound",
	  sp->fullname,
	  sp->syncing,
	  sp->contrasting,sp->contrast,
	  sp->expanding,sp->expand);
  write(fd,save_buf,strlen(save_buf));
  sprintf(save_buf,"  (set-speed %.4f)\n  (set-reverbing %d)\n  (set-reverb-scale %.4f)\n  (set-reverb-length %.4f)\n  (set-amp %.4f)\n",
	  sp->srate,sp->reverbing,sp->revscl,sp->revlen,sp->amp);
  write(fd,save_buf,strlen(save_buf));
  sprintf(save_buf,"  (set-filtering %d)\n  (set-filter_order %d)\n",
	  sp->filtering,sp->filter_order);
  write(fd,save_buf,strlen(save_buf));
  for (chan=0;chan<sp->nchans;chan++)
    {
      cp = sp->chans[chan];
      ap = cp->axis;
      sprintf(save_buf,"  (select-channel %d)\n    (set-waving %d) (set-ffting %d) (set-x-bounds %.4f %.4f) (set-y-bounds %.4f %.4f) (set-cursor %d)\n",
	      chan,
	      cp->waving,
	      cp->ffting,
	      ap->x0,ap->x1,ap->y0,ap->y1,cp->cursor);
      write(fd,save_buf,strlen(save_buf));
    }
  return(0);
}

void set_session_name (snd_state *ss, char *name)
{
  dir *dp;
  int i,latest,fd;
  time_t latest_date,date;
  char *filename,*sp,*dot;
  if (strcmp(name,".") == 0)
    {
      /* search for latest in *.snd-session list on current directory -- if none complain */
      dp = find_session_files_in_dir(".");
      if (dp->len == 0)
	fprintf(stderr,snd_string_no_session_files_found);
      else
	if (dp->len == 1)
	  set_session_file(ss,copy_string(dp->files[0]));
	else
	  {  
	    latest_date = file_write_date(dp->files[0]);
	    latest = 0;
	    for (i=1;i<dp->len;i++)
	      {
		date = file_write_date(dp->files[i]);
		if (date > latest_date) {latest_date = date; latest = i;}
	      }
	    set_session_file(ss,copy_string(dp->files[latest]));
	  }
      free_dir(dp);
    }
  else 
    {
      /* check first for .snd-session extension */
      dot = NULL;
      for (sp=name;(*sp) != '\0';sp++) if ((*sp) == '.') dot=sp;
      if ((dot) && (strcmp(dot,".snd-session") == 0))
	set_session_file(ss,copy_string(name));
      else
	{
	  filename = (char *)calloc(snd_strlen(name)+13,sizeof(char));
	  sprintf(filename,"%s.snd-session",name);
	  fd = open(filename,O_RDONLY,0);
	  if (fd != -1)
	    {
	      set_session_file(ss,copy_string(filename));
	      close(fd);
	      return;
	    }
	  else
	    {
	      free(filename);
	      set_session_file(ss,copy_string(name));
	    }
	}
      fd = open(session_file(ss),O_RDONLY,0);
      if (fd == -1) 
	{
	  fprintf(stderr,snd_string_cant_find,session_file(ss));
	  free(session_file(ss));
	  set_session_file(ss,NULL);
	}
      else close(fd);
    }
}

void save_state (snd_state *ss, char *session_name)
{
  int save_fd;
  save_fd = open_restart_file(session_name,FALSE);
  if (save_fd == -1) 
    {
      sprintf(save_buf,snd_string_cant_write_p,session_name,strerror(errno));
      snd_printf(ss,save_buf);
    }
  else
    {
      save_snd_state_options(ss,save_fd);
      map_over_sounds(ss,save_sound_state,(void *)(&save_fd));  /* current sound state -- will traverse chans */
      sprintf(save_buf,"(set-window-height %d) (set-window-width %d)\n",
	      snd_window_height(ss),snd_window_width(ss));
      write(save_fd,save_buf,strlen(save_buf));
      save_dialog_state(ss,save_fd);                         /* current dialog state not covered above */
      save_macro_state(ss,save_fd);                          /* current unsaved keyboard macros (snd-chn.c) */
      save_recorder_state(ss,save_fd);                       /* current recorder window state */
      save_envelope_editor_state(ss,save_fd);                /* current envelope editor window state */
      close(save_fd);
    }
}


void restore_state (snd_state *ss, char *session_name)
{
  snd_load_file(ss,session_name);
}
